/*
    Copyright (c) 2019 David Edmundson <davidedmundson@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/
#pragma once

#include "SensorObject.h"
#include "SensorPlugin.h"
#include "SensorProperty.h"

#include <QScopedPointer>
class QProcess;
class GPU;

class NvidiaPlugin : public SensorPlugin
{
    Q_OBJECT
public:
    NvidiaPlugin(QObject *parent, const QVariantList &args);
    QString providerName() const override
    {
        return QStringLiteral("nvidia");
    }

    void init();

private:
    void gpuSubscriptionChanged(bool subscribed);

    QProcess *m_process;
    QHash<int, GPU *> m_gpus;
    int m_activeWatcherCount = 0;
};
